/*SQL-Skript zur Erzeugung verschiedener Integer-Spalten,
die QGIS für die regelbasierte Symbolsierung auswerten kann.
In diesen Spalten gibt es nur 1 oder 0, so dass keine komplexen Abfragen
beim zoomen und laden durchgeführt werden müssen.
*/

Begin Transaction;

alter table multipolygons 
add column wohngebiet integer default 0;
alter table multipolygons
add column industrie_gewerbe integer default 0;
alter table multipolygons 
add column einkauf integer default 0;
alter table multipolygons 
add column gebaeude integer default 0;
alter table multipolygons 
add column wichtige_gebaeude integer default 0;
alter table multipolygons 
add column gruenflaechen integer default 0;
alter table multipolygons 
add column friedhof integer default 0;
alter table multipolygons 
add column park integer default 0;
alter table multipolygons 
add column garten_obst integer default 0;
alter table multipolygons 
add column wald integer default 0;
alter table multipolygons 
add column gewaesser integer default 0;
alter table multipolygons 
add column grenze integer default 0;
alter table multipolygons 
add column gegrenze integer default 0;
alter table multipolygons 
add column hnr integer default 0;


/*Spalten für die Darstellung der Flächennutzung füllen.
Die einzelnen Updates wenden die Abfragen an, welche ursprünglich
für die regelbasierte Darstellung direkt im QGIS eingegeben wurden.
Anschließend muss QGIS nur Noch "Spalte = 1" auswerten
*/
update  multipolygons 
set
wohngebiet = 1
where "landuse" = 'residential';

update  multipolygons 
set
industrie_gewerbe = 1
where "landuse" IN ( 'industrial', 'commercial' , 'industrial' ,'retail');

update  multipolygons 
set
einkauf = 1
where "landuse" = 'retail';

update  multipolygons 
set
gebaeude = 1
where building IS NOT NULL;

update  multipolygons 
set
wichtige_gebaeude = 1
where building is NOT NULL and
 ("building" IN ( 'school' , 'hospital' , 'church' , 'train_station' ) OR  
"amenity" IN ( 'hospital' , 'kindergarten' , 'place_of_worship' , 'public_building' , 'school' ) OR   "name" IS NOT NULL);

update  multipolygons 
set
gruenflaechen = 1
where "leisure" in ( 'park', 'pitch','playground', 'garden') OR  "landuse" in ( 'recreation_ground','cemetery', 'orchard' , 'quarry' , 'vineyard' ,'allotments','village_green');

update  multipolygons 
set
friedhof = 1
where "landuse" = 'cemetery';

update  multipolygons 
set
park = 1
where "leisure" in ( 'park', 'pitch','playground') OR  "landuse" = 'recreation_ground';

update  multipolygons 
set
garten_obst = 1
where  "leisure" = 'garden' OR  "landuse" IN ( 'orchard' , 'quarry' , 'vineyard' ,'allotments');

update  multipolygons 
set
wald = 1
where  "landuse" = 'forest' OR  "natural"  =  'wood';

update  multipolygons 
set
gewaesser = 1
where "landuse" in ( 'basin' , 'reservoir') OR "natural"  =  'water' 
OR  "leisure"  =  'swimming_pool'  
OR  "waterway" in ( 'riverbank' , 'stream' , 'water' , 'lake' , 'water' );

update  multipolygons 
set
grenze = 1
where  "admin_level" in ( '8' , '9');

update  multipolygons 
set
gegrenze = 1
where  "admin_level" in ( '8');

update  multipolygons 
set
hnr = 1
where  addr_housenumber is not NULL;



-- Spalten für die Darstellung der Straßen anlegen.
alter table lines
add column autobahn integer default 0;
alter table lines
add column bundesstrasse integer default 0;
alter table lines
add column landesstrasse integer default 0;
alter table lines
add column kreisstrasse integer default 0;
alter table lines
add column wohnstrasse integer default 0;
alter table lines
add column fussgaengerzone integer default 0;
alter table lines
add column fussweg integer default 0;
alter table lines
add column sonstige_hauptstr integer default 0;
alter table lines
add column strasse integer default 0;
alter table lines
add column ha_strasse integer default 0;
alter table lines
add column treppe integer default 0;


update  lines
set
autobahn = 1
where   "highway" in ( 'motorway' , 'motorway_link' );


update  lines
set
bundesstrasse = 1
where    "highway"  in( 'primary' , 'primary_link', 'trunk', 'trunk_link' );


update  lines
set
landesstrasse = 1
where    "highway" in( 'secondary' , 'secondary_link' );


update  lines
set
kreisstrasse = 1
where    "highway" = 'tertiary' ;


update  lines
set
wohnstrasse = 1
where     "highway" in (  'residential' , 'road' , 'unclassified', 'service');


update  lines
set
fussgaengerzone = 1
where   "highway"  in ( 'pedestrian','living_street') ;


update  lines
set
fussweg = 1
where    "highway" in(  'track' , 'track; path', 'path' , 'footway' ,'cycleway');

update  lines
set
treppe = 1
where    "highway"= 'steps';


update  lines
set
sonstige_hauptstr = 1
where    "highway" in ( 'secondary' , 'secondary_link', 'tertiary' );


update  lines
set
strasse = 1
where    "highway" in ( 'living_street' ,  'pedestrian' , 'residential' , 'road' , 'service',  'unclassified' );


update  lines
set
ha_strasse = 1
where    "highway" in ('primary' , 'primary_link', 'trunk', 'trunk_link', 'secondary' , 'secondary_link', 'tertiary' );

--Spalten fuer die Symbolisierung des Schienenetzes anlegen
alter table lines
add column eisb_gr integer default 0;
alter table lines
add column eisb_gr_tu integer default 0;
alter table lines
add column eisb_kl integer default 0;
alter table lines
add column eisb_kl_tu integer default 0;
alter table lines
add column strab integer default 0;
alter table lines
add column ubahn integer default 0;

-- Spalten zur Symbolsierung des Schienennetzes füllen
-- Eisenbahnen großer Maßstab ohne Tunnel
update  lines
set
eisb_gr = 1
where  "railway" =  'rail' and  ( "tunnel" is NULL or  "tunnel" in ( 'no' ));

update  lines
set
eisb_gr_tu = 1
where  "railway" = 'rail'  and  "tunnel" in ( 'building_passage' , 'culvert' , 'yes' );

update  lines
set
eisb_kl = 1
where  "railway" =  'rail' and  "name" is not null and ( "tunnel" is NULL or  "tunnel" in ( 'no' ));

update  lines
set
eisb_kl_tu = 1
where  "railway" =  'rail' and  "name" is not null and  "tunnel"  in ( 'building_passage' , 'culvert' , 'yes' );

update  lines
set
strab = 1
where   ("railway" = 'tram' or  "railway" = 'light_rail') and ("tunnel" is NULL or "tunnel" = 'no');

update  lines
set
ubahn = 1
where  "railway" =  'subway' or ("railway" = 'light_rail' and "tunnel"  in ( 'building_passage' , 'culvert' , 'yes')) ;


-- Indexe erstellen für perfomante Abfragen
create index wg on multipolygons (wohngebiet);
create index ig on multipolygons (industrie_gewerbe);
create index ek on multipolygons (einkauf);
create index ge on multipolygons (gebaeude);
create index wg2 on multipolygons (wichtige_gebaeude);
create index gr on multipolygons (gruenflaechen);
create index fh on multipolygons (friedhof);
create index pa on multipolygons (park);
create index gao on multipolygons (garten_obst);
create index wal on multipolygons (wald);
create index gew on multipolygons (gewaesser);
create index gre on multipolygons (grenze);
create index geg on multipolygons (gegrenze);
create index hnr on multipolygons (hnr);

create index au on  lines (autobahn);
create index bu on  lines (bundesstrasse);
create index las on  lines (landesstrasse);
create index krs on  lines (kreisstrasse);
create index wos on  lines (wohnstrasse);
create index fussg on  lines (fussgaengerzone);
create index fuw on  lines (fussweg);
create index so on  lines (sonstige_hauptstr);
create index str on  lines (strasse);
create index hastr on  lines (ha_strasse);
create index eis on  lines (eisb_gr);
create index ei2 on  lines (eisb_gr_tu);
create index eis3 on  lines (eisb_kl);
create index eis4 on  lines (eisb_kl_tu);
create index strab on  lines (strab);
create index uba on  lines (ubahn);
create index trepp on  lines (treppe);

create index nami on  lines (name);
create index namipo on multipolygons (name);

commit;

Begin Transaction;
/*Hausnummertabelle
Hausnummer für Punktlayer anlegen und mit den Polygonhausnummern vereinen */

-- poi mit hausnummern kennzeichnen / Spalte anlegen
alter table points  add column hnr integer default 0 ;

-- Tabelle poi mit haus ummern kennzeichnen
update  points 
set
hnr = 1
where  addr_housenumber is not NULL;

create index adrh on points (hnr);

/*Die POIs mit Hasunummern auswählen, die auf Gebäuden ohne Hnr-Attribut liegen
Poihausnummern und Gebäudehaunummern zusammenfassen */


create table tbl_hnr as


select 
a.geometry as geometry,		-- Punkte mit Attribut addr_housenumber as tbl_point
a.addr_housenumber as hausnr
from points as a inner join 
	-- Unterabfrage wählt Gebäude ohne Hausnummern
	(select
	geometry
	from multipolygons
	where gebaeude = 1 and hnr = 0) as b

 on st_intersects(b.geometry, buffer(a.geometry,1)) 	-- Die Punkte müssen auf gebäuden liegen (mit Toleranz 1m)
where  hnr = 1	
						-- und Sie müssen eine Hasunummer haben and

and
a.ROWID In (SELECT ROWID FROM SpatialIndex WHERE f_table_name='points' AND search_frame=b.geometry)


union								-- Vereinigung mit den den vorhandenen Gebäudehausnummern

select
st_pointonsurface(geometry) as geometry,	-- Aus Gebäudegeometry einen Punkt generieren
addr_housenumber as hausnr
from multipolygons

where hnr = 1;

create index t_hnr on tbl_hnr (hausnr);

select RecoverGeometryColumn('tbl_hnr','Geometry',25832,'POINT');
Select CreateSpatialIndex('tbl_hnr','Geometry');
SELECT UpdateLayerStatistics('tbl_hnr');

commit;

Begin Transaction;

--bushaltr
create table bushaltestellen as
SELECt
geometry as geometry,
name,
highway,
case when wheelchair is not null or wheelchair  != 'no' then 'ja' else 'nein' end as barrierefrei,
case when shelter is not null or shelter  != 'no' then 'ja' else 'nein' end as ueberdacht

from points
where highway = 'bus_stop';

Select RecoverGeometryColumn('bushaltestellen','Geometry',25832,'POINT');
Select CreateSpatialIndex('bushaltestellen','Geometry');
SELECT UpdateLayerStatistics('bushaltestellen');


/*
create table leitungen as
select
ogc_fid as id,
geometry as geometry,
power as typ,
voltage,
cables,
other_tags as sonstiges
from lines
where power in ('line', 'minor_line');

Select RecoverGeometryColumn('leitungen','Geometry',25832,'LINESTRING');
Select CreateSpatialIndex('leitungen','Geometry');
SELECT UpdateLayerStatistics('leitungen');
*/

create table buslinien as
SELECT 
casttomultilinestring(_ogr_geometry_) as geometry,
name as name,
type as type,
other_tags as other_tags
FROM multilinestrings
where (name like '%Bus%' or other_tags like '%Bus%') and not  name like '%rund um%';

Select RecoverGeometryColumn('buslinien','Geometry',25832,'MULTILINESTRING');
Select CreateSpatialIndex('buslinien','Geometry');
SELECT UpdateLayerStatistics('buslinien');




create table schulen as
SELECt
centroid(geometry) as geometry,
name,
building,
leisure,
amenity
FROM multipolygons
where (name like '%schule%' or name like 'Schule%' or amenity = 'school'  or building = 'school') and not name like 'Baum%' 
and not name like 'Fahr%' 
and not name like 'Tanz%';

Select RecoverGeometryColumn('schulen','Geometry',25832,'POINT');
Select CreateSpatialIndex('schulen','Geometry');
SELECT UpdateLayerStatistics('schulen');

create table krankenhaus as
SELECt
centroid(geometry) as geometry,
name,
building,
leisure,
amenity
FROM multipolygons
where (name like '%klinik%' or name like '%krankenhaus%' or amenity like '%hospit%l%' );

Select RecoverGeometryColumn('krankenhaus','Geometry',25832,'POINT');
Select CreateSpatialIndex('krankenhaus','Geometry');
SELECT UpdateLayerStatistics('krankenhaus');


create table bahn_halte as

select

name,
public_transport,
shelter as dach,
wheelchair as barrierefrei,
geometry

from points

where public_transport is not null and highway is NULL
;

Select RecoverGeometryColumn('bahn_halte','Geometry',25832,'POINT');
Select CreateSpatialIndex('bahn_halte','Geometry');
SELECT UpdateLayerStatistics('bahn_halte');



--gemneindegrenzen
create table gemeindegrenzen(
id INTEGER PRIMARY KEY AUTOINCREMENT,
name varchar,
geometry geometry);

insert into gemeindegrenzen(
name,
geometry)
select
name,
geometry
from multipolygons
where admin_level = 8
;

Select RecoverGeometryColumn('gemeindegrenzen','Geometry',25832,'MULTIPOLYGON');
Select CreateSpatialIndex('gemeindegrenzen','Geometry');
SELECT UpdateLayerStatistics('gemeindegrenzen');

create table plz(
id INTEGER PRIMARY KEY AUTOINCREMENT,
name varchar,
postal_code varchar,
geometry geometry);

insert into plz(
name,
postal_code,
geometry)
select
name,
postal_code,
geometry
from multipolygons
where postal_code is not null;
;

Select RecoverGeometryColumn('plz','Geometry',25832,'MULTIPOLYGON');
Select CreateSpatialIndex('plz','Geometry');
SELECT UpdateLayerStatistics('plz');

commit;


Begin Transaction;


/*
create table buslinien_diss as
Select
1 as id,
Gunion(Geometry) as geometry
from buslinien
group by NULL;

Select RecoverGeometryColumn('buslinien_diss','Geometry',25832,'multilinestring');
Select CreateSpatialIndex('buslinien_diss','Geometry');
SELECT UpdateLayerStatistics('buslinien_diss');

--

select ElementaryGeometries('buslinien_diss','Geometry','buslinien_diss_ele','pk_id','multi_id');


Select RecoverGeometryColumn('buslinien_diss_ele','Geometry',25832,'linestring');
Select CreateSpatialIndex('buslinien_diss_ele','Geometry');
SELECT UpdateLayerStatistics('buslinien_diss_ele');

--------------------------
-----------------------------


-------------

create table bus_attrib as

select distinct
a.pk_id as id,
a.geometry as geometry,
count(b.ROWID) as anzahl,
replace(group_concat(distinct b.name),',','; ') as linienliste
from buslinien_diss_ele as a inner join buslinien as b on ST_Within(a.geometry,b.geometry)
where a.ROWID IN
            (SELECT ROWID FROM SpatialIndex WHERE f_table_name='buslinien_diss_ele' AND search_frame=b.geometry)
group by a.pk_id
;

-----------
create table bus_attrib as

	with bussegmente as
		(
		select distinct
		b.ROWID as bid,
		a.pk_id as id,
		a.geometry as geometry,
		b.name as name
		from buslinien_diss_ele as a inner join buslinien as b on ST_Within(a.geometry,b.geometry)
		where a.ROWID IN
			    (SELECT ROWID FROM SpatialIndex WHERE f_table_name='buslinien_diss_ele' AND search_frame=b.geometry)
		)
	select distinct
	a.id as id,
	a.geometry as geometry,
	count(a.bid) as anzahl,
	replace(group_concat(distinct a.name),',','; ') as linienliste
	from bussegmente as a
	group by a.id
;
---------------------


--
create table buslinie_grupp as
select distinct
casttomultilinestring(gunion(geometry)) as geometry,
anzahl as anzahl,
linienliste as linienliste
from bus_attrib
group by linienliste
;
Select RecoverGeometryColumn('buslinie_grupp','Geometry',25832,'MULTILINESTRING');
Select CreateSpatialIndex('buslinie_grupp','Geometry');
SELECT UpdateLayerStatistics('buslinie_grupp');

select dropgeotable('bus_attrib');

select dropgeotable('buslinien_diss');

select dropgeotable('buslinien_diss_ele');

commit;

vacuum;
*/

--Stadtbahn1

create table tramlinien as

SELECT 
_ogr_geometry_ as geometry,
name as name,
type as type,
other_tags as other_tags
FROM multilinestrings
where (name like '%tram%' or other_tags like '%tram%' or name like '%subway%' or other_tags like '%subway%');


Select RecoverGeometryColumn('tramlinien','Geometry',25832,'MULTILINESTRING');
Select CreateSpatialIndex('tramlinien','Geometry');
SELECT UpdateLayerStatistics('tramlinien');


create table stadtbahn as

select
a.ogc_fid as ogc_fid,
a.name as  nam,
a.railway as railway,
a.geometry as geometry
from lines as a inner join tramlinien as b on st_intersects(a.geometry,buffer(b.geometry,1))

where   "railway" in ( 'tram','light_rail','rail','subway')
   and  b.ROWID IN
            (SELECT ROWID FROM SpatialIndex WHERE f_table_name='tramlinien' AND search_frame=a.geometry);


Select RecoverGeometryColumn('stadtbahn','Geometry',25832,'linestring');
Select CreateSpatialIndex('stadtbahn','Geometry');
SELECT UpdateLayerStatistics('stadtbahn');

----
select ElementaryGeometries('stadtbahn','Geometry','stadtbahn_ele','pk_id','multi_id');
Select RecoverGeometryColumn('stadtbahn_ele','Geometry',25832,'linestring');

Select CreateSpatialIndex('stadtbahn_ele','Geometry');


--create table bus_attrib as


-------------

create table tram_attrib as
select
a.pk_id as id,
a.geometry as geometry,
count(distinct b.name) as anzahl,
replace((group_concat(distinct b.name)),',','; ') as linienliste
from stadtbahn_ele as a inner join tramlinien as b on ST_Within(a.geometry,buffer(b.geometry,1))
where b.ROWID IN
            (SELECT ROWID FROM SpatialIndex WHERE f_table_name='tramlinien' AND search_frame=a.geometry)
group by a.pk_id
;

--
create table tramlinien_grupp as
select distinct
casttomultilinestring(gunion(geometry)) as geometry,
anzahl as anzahl,
linienliste as linienliste
from tram_attrib
group by linienliste
;
Select RecoverGeometryColumn('tramlinien_grupp','Geometry',25832,'MULTILINESTRING');
Select CreateSpatialIndex('tramlinien_grupp','Geometry');
SELECT UpdateLayerStatistics('tramlinien_grupp');


select dropgeotable('tram_attrib');


select dropgeotable('stadtbahn_ele');

delete from spatial_ref_sys_aux where srid <> 25832;
delete from spatial_ref_sys where auth_srid <> 25832;

vacuum;

