/*SQL-Skript zur Erzeugung verschiedener Integer-Spalten,
die QGIS für die regelbasierte Symbolsierung auswerten kann.
In diesen Spalten gibt es nur 1 oder 0, so dass keine komplexen Abfragen
beim zoomen und laden durchgeführt werden müssen.
*/

-- Spalten für die Darstellung der Flächennutzung anlegen.
alter table osm.tbl_landuse 
add column wohngebiet integer default 0,
add column industrie_gewerbe integer default 0,
add column einkauf integer default 0,
add column gebaeude integer default 0,
add column wichtige_gebaeude integer default 0,
add column gruenflaechen integer default 0,
add column friedhof integer default 0,
add column park integer default 0,
add column garten_obst integer default 0,
add column wald integer default 0,
add column gewaesser integer default 0,
add column grenze integer default 0,
add column gegrenze integer default 0,
add column hnr integer default 0,
add column brunnen integer default 0;



/*Spalten für die Darstellung der Flächennutzung füllen.
Die einzelnen Updates wenden die Abfragen an, welche ursprünglich
für die regelbasierte Darstellung direkt im QGIS eingegeben wurden.
Anschließend muss QGIS nur Noch "Spalte = 1" auswerten
*/
update  osm.tbl_landuse 
set
wohngebiet = 1
where "landuse" = 'residential';

update  osm.tbl_landuse 
set
industrie_gewerbe = 1
where "landuse" IN ( 'industrial', 'commercial' , 'industrial' ,'retail');

update  osm.tbl_landuse 
set
einkauf = 1
where "landuse" = 'retail';

update  osm.tbl_landuse 
set
gebaeude = 1
where building IS NOT NULL;

update  osm.tbl_landuse 
set
wichtige_gebaeude = 1
where building is NOT NULL and
 ("building" IN ( 'school' , 'hospital' , 'church' , 'train_station' ) OR  
"amenity" IN ( 'hospital' , 'kindergarten' , 'place_of_worship' , 'public_building' , 'school' ) OR   "name" IS NOT NULL);

update  osm.tbl_landuse 
set
gruenflaechen = 1
where "leisure" in ( 'park', 'pitch','playground', 'garden') OR  "landuse" in ( 'recreation_ground','cemetery', 'orchard' , 'quarry' , 'vineyard' ,'allotments');

update  osm.tbl_landuse 
set
friedhof = 1
where "landuse" = 'cemetery';

update  osm.tbl_landuse 
set
park = 1
where "leisure" in ( 'park', 'pitch','playground') OR  "landuse" = 'recreation_ground';

update  osm.tbl_landuse 
set
garten_obst = 1
where  "leisure" = 'garden' OR  "landuse" IN ( 'orchard' , 'quarry' , 'vineyard' ,'allotments');

update  osm.tbl_landuse 
set
wald = 1
where  "landuse" = 'forest' OR  "natural"  =  'wood';

update  osm.tbl_landuse 
set
gewaesser = 1
where "landuse" = 'basin' OR "natural"  =  'water' 
OR  "leisure"  =  'swimming_pool'  
OR  "waterway" in ( 'riverbank' , 'stream' , 'water' , 'lake' , 'water' );

update  osm.tbl_landuse 
set
grenze = 1
where  "admin_level" in ( '8' , '9');

update  osm.tbl_landuse 
set
gegrenze = 1
where  "admin_level" in ( '8');

update  osm.tbl_landuse 
set
hnr = 1
where  addr_housenumber is not NULL;

update osm.tbl_landuse 
set
brunnen = 1
where "landuse" = 'basin' or "amenity" = 'fountain';




-- Spalten für die Darstellung der Straßen anlegen.
alter table osm.tbl_linien
add column autobahn integer default 0;
alter table osm.tbl_linien
add column bundesstrasse integer default 0;
alter table osm.tbl_linien
add column landesstrasse integer default 0;
alter table osm.tbl_linien
add column kreisstrasse integer default 0;
alter table osm.tbl_linien
add column fahrweg integer default 0;
alter table osm.tbl_linien
add column wohnstrasse integer default 0;
alter table osm.tbl_linien
add column fussgaengerzone integer default 0;
alter table osm.tbl_linien
add column fussweg integer default 0;
alter table osm.tbl_linien
add column sonstige_hauptstr integer default 0;
alter table osm.tbl_linien
add column strasse integer default 0;
alter table osm.tbl_linien
add column ha_strasse integer default 0;
alter table osm.tbl_linien
add column treppe integer default 0;


update  osm.tbl_linien
set
autobahn = 1
where   "highway" in ( 'motorway' , 'motorway_link' );


update  osm.tbl_linien
set
bundesstrasse = 1
where    "highway"  in( 'primary' , 'primary_link', 'trunk', 'trunk_link' );


update  osm.tbl_linien
set
landesstrasse = 1
where    "highway" in( 'secondary' , 'secondary_link' );


update  osm.tbl_linien
set
kreisstrasse = 1
where    "highway" = 'tertiary' ;


update  osm.tbl_linien
set
wohnstrasse = 1
where     "highway" in (  'residential' , 'road' , 'unclassified', 'service');


update  osm.tbl_linien
set
fussgaengerzone = 1
where   "highway"  in ( 'pedestrian','living_street') ;


update  osm.tbl_linien
set
fussweg = 1
where    "highway" in(  'track' , 'track; path', 'path' , 'footway' ,'cycleway') and (tracktype not in ('grade1')) or tracktype is NULL;

update  osm.tbl_linien
set
fahrweg = 1
where    "highway" =  'track' and tracktype in ('grade1');

update  osm.tbl_linien
set
treppe = 1
where    "highway"= 'steps';


update  osm.tbl_linien
set
sonstige_hauptstr = 1
where    "highway" in ( 'secondary' , 'secondary_link', 'tertiary' );


update  osm.tbl_linien
set
strasse = 1
where    "highway" in ( 'living_street' ,  'pedestrian' , 'residential' , 'road' , 'service',  'unclassified' );


update  osm.tbl_linien
set
ha_strasse = 1
where    "highway" in ('primary' , 'primary_link', 'trunk', 'trunk_link', 'secondary' , 'secondary_link', 'tertiary' );

--Spalten fuer die Symbolisierung des Schienenetzes anlegen
alter table osm.tbl_linien
add column eisb_gr integer default 0;
alter table osm.tbl_linien
add column eisb_gr_tu integer default 0;
alter table osm.tbl_linien
add column eisb_kl integer default 0;
alter table osm.tbl_linien
add column eisb_kl_tu integer default 0;
alter table osm.tbl_linien
add column strab integer default 0;
alter table osm.tbl_linien
add column ubahn integer default 0;

-- Spalten zur Symbolsierung des Schienennetzes füllen
-- Eisenbahnen großer Maßstab ohne Tunnel
update  osm.tbl_linien
set
eisb_gr = 1
where  "railway" =  'rail' and  ( "tunnel" is NULL or  "tunnel" in ( 'no' ));

update  osm.tbl_linien
set
eisb_gr_tu = 1
where  "railway" = 'rail'  and  "tunnel" in ( 'building_passage' , 'culvert' , 'yes' );

update  osm.tbl_linien
set
eisb_kl = 1
where  "railway" =  'rail' and  "name" is not null and ( "tunnel" is NULL or  "tunnel" in ( 'no' ));

update  osm.tbl_linien
set
eisb_kl_tu = 1
where  "railway" =  'rail' and  "name" is not null and  "tunnel"  in ( 'building_passage' , 'culvert' , 'yes' );

update  osm.tbl_linien
set
strab = 1
where   ("railway" = 'tram' or  "railway" = 'light_rail') and ("tunnel" is NULL or "tunnel" = 'no');

update  osm.tbl_linien
set
ubahn = 1
where  "railway" =  'subway' or ("railway" = 'light_rail' and "tunnel"  in ( 'building_passage' , 'culvert' , 'yes')) ;

-- Indexe erstellen für perfomante Abfragen
create index on osm.tbl_landuse (wohngebiet);
create index on osm.tbl_landuse (industrie_gewerbe);
create index on osm.tbl_landuse (einkauf);
create index on osm.tbl_landuse (gebaeude);
create index on osm.tbl_landuse (wichtige_gebaeude);
create index on osm.tbl_landuse (gruenflaechen);
create index on osm.tbl_landuse (friedhof);
create index on osm.tbl_landuse (park);
create index on osm.tbl_landuse (garten_obst);
create index on osm.tbl_landuse (wald);
create index on osm.tbl_landuse (gewaesser);
create index on osm.tbl_landuse (grenze);
create index on osm.tbl_landuse (gegrenze);
create index on osm.tbl_landuse (hnr);

create index on  osm.tbl_linien (autobahn);
create index on  osm.tbl_linien (bundesstrasse);
create index on  osm.tbl_linien (landesstrasse);
create index on  osm.tbl_linien (kreisstrasse);
create index on  osm.tbl_linien (wohnstrasse);
create index on  osm.tbl_linien (fussgaengerzone);
create index on  osm.tbl_linien (fussweg);
create index on  osm.tbl_linien (sonstige_hauptstr);
create index on  osm.tbl_linien (strasse);
create index on  osm.tbl_linien (ha_strasse);
create index on  osm.tbl_linien (eisb_gr);
create index on  osm.tbl_linien (eisb_gr_tu);
create index on  osm.tbl_linien (eisb_kl);
create index on  osm.tbl_linien (eisb_kl_tu);
create index on  osm.tbl_linien (strab);
create index on  osm.tbl_linien (ubahn);
create index on  osm.tbl_linien (treppe);

create index on  osm.tbl_linien (name);
create index on osm.tbl_landuse (name);
create index on osm.tbl_landuse (brunnen);


/*Hausnummertabelle
Hausnummer für Punktlayer anlegen und mit den Polygonhausnummern vereinen */

-- poi mit hausnummern kennzeichnen / Spalte anlegen
alter table osm.tbl_poi  add column hnr integer default 0 ;

-- Tabelle poi mit haus ummern kennzeichnen
update  osm.tbl_poi 
set
hnr = 1
where  addr_housenumber is not NULL;

create index on osm.tbl_poi (hnr);

/*Die POIs mit Hasunummern auswählen, die auf Gebäuden ohne Hnr-Attribut liegen
Poihausnummern und Gebäudehaunummern zusammenfassen */


create table osm.tbl_hnr as

-- Hausnummerpunkte extrahieren, die auf Gebäude ohne Hausnummern ligen
select 
(a.wkb_geometry)::geometry('POINT',25832) as geometry,		-- Punkte mit Attribut addr_housenumber as tbl_point
a.addr_housenumber as hausnr
from osm.tbl_poi as a inner join 
	-- Unterabfrage wählt Gebäude ohne Hausnummern
	(select
	wkb_geometry
	from osm.tbl_landuse
	where gebaeude = 1 and hnr = 0) as b

 on st_intersects(b.wkb_geometry, a.wkb_geometry) 	-- Die Punkte müssen auf gebäuden liegen)
 where hnr = 1							-- und Sie müssen eine Hasunummer haben

union								-- Vereinigung mit den den vorhandenen Gebäudehausnummern

select
st_pointonsurface(wkb_geometry)::geometry('POINT',25832) as geometry,	-- Aus Gebäudegeometry einen Punkt generieren
addr_housenumber as hausnr
from osm.tbl_landuse

where hnr = 1;							-- Nur Gebäude mit Hausnummer

-- Primärschlüssel anlegen
Alter table osm.tbl_hnr
add column gids serial NOT NULL;

ALTER TABLE  osm.tbl_hnr
  ADD CONSTRAINT hnr_prim PRIMARY KEY(gids);

--KBS definieren
select UpdateGeometrySRID('osm', 'tbl_hnr', 'geometry',25832);

--Indexe anlegen
CREATE INDEX tbl_hnr_gix ON osm.tbl_hnr USING GIST (geometry);

create index on osm.tbl_hnr (hausnr);




---Brunnen
create table osm.brunnen as

select
wkb_geometry as geometry,
name as name
from osm.tbl_poi
where amenity = 'fountain'

union

select
st_pointonsurface(wkb_geometry)::geometry('POINT') as geometry,
name as name
from osm.tbl_landuse
where brunnen = 1;

Alter table osm.brunnen
add column gids serial NOT NULL;

ALTER TABLE  osm.brunnen
  ADD CONSTRAINT brunnen_prim PRIMARY KEY(gids);

select UpdateGeometrySRID('osm', 'brunnen', 'geometry',25832);

CREATE INDEX brunnen_gix ON osm.brunnen USING GIST (geometry);





